<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet
  xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
  xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
  version="1.0">
  <xsl:output method="xml"/>
  <xsl:strip-space elements="*"/>

<xsl:template match="/certificationProblem">
  <!-- Include stylesheet information in output -->
  <xsl:text disable-output-escaping="yes">
    &lt;?xml-stylesheet type="text/xsl" href="cpfHTML.xsl"?&gt;
  </xsl:text>
  <xsl:copy>  
      <xsl:attribute name="xsi:noNamespaceSchemaLocation">cpf.xsd</xsl:attribute>
      <xsl:apply-templates select="input"/>
      <cpfVersion>2.0</cpfVersion>
      <xsl:apply-templates select="proof"/>
      <xsl:apply-templates select="origin"/>
  </xsl:copy>        
</xsl:template>

<!-- all nodes that should be wrapped inside trs(Non)TerminationProof -->
<xsl:template match="flatContextClosure|rIsEmpty|ruleRemoval|semlab|uncurry|unlab|variableConditionViolated">
  <xsl:choose>
    <xsl:when test="//certificationProblem/goal/termination">
      <trsTerminationProof>
        <xsl:copy><xsl:apply-templates/></xsl:copy>
      </trsTerminationProof>
    </xsl:when>
    <xsl:when test="//certificationProblem/goal/nonTermination">
      <trsNonterminationProof>
        <xsl:copy><xsl:apply-templates/></xsl:copy>
      </trsNonterminationProof>
    </xsl:when>
  </xsl:choose>
</xsl:template>

<!-- all nodes that should be wrapped inside dpProof -->
<xsl:template match="argumentFilterProc|depGraphProc|flatContextClosureProc|pIsEmpty|semlabProc|uncurryProc|unlabProc">
  <dpProof>
    <xsl:copy><xsl:apply-templates/></xsl:copy>
  </dpProof>
</xsl:template>

<!-- all nodes that should be wrapped inside orderingConstraintProof -->
<xsl:template match="redPair">
  <orderingConstraintProof>
    <xsl:copy><xsl:apply-templates/></xsl:copy>
  </orderingConstraintProof>
</xsl:template>

<!-- all nodes that should be copied -->
<xsl:template match="arg|argumentFilter|argumentFilterEntry|after|before|box|coefficient|component|domain|dps|edge|etaRules|flatContexts|freshSymbol|funapp|funContext|input|interpret|interpretation|lhs|loop|matrices|matrix|matrixinterpretation|model|nonCollapsing|pathOrder|polynomial|product|projEntry|proof|rationals|rhs|rule|rules|simpleProjection|sizeChangeGraph|statusPrecedence|statusPrecedenceEntry|substEntry|sum|trs|trsInput|type|uncurriedSymbolEntry|uncurriedSymbols|uncurryInformation|usableRules|vector|redex|positionInTerm|substitution">
  <xsl:copy><xsl:apply-templates/></xsl:copy>
</xsl:template>

<!-- all nodes for which even the child nodes may be copied -->
<xsl:template match="applicativeTop|argument|arity|collapsing|degree|denominator|dimension|integer|lex|minusInfinity|markedSymbols|naturals|number|numerator|open|position|precedence|rational|realScc|rootLabeling|strict|strictDimension|subtermCriterion|var|variable">
  <xsl:copy-of select="."/>
</xsl:template>

<xsl:template match="arctic">
  <arctic><domain><naturals/></domain></arctic>
</xsl:template>

<xsl:template match="arcticBelowZero">
  <arctic><domain><integers/></domain></arctic>
</xsl:template>

<xsl:template match="rationals">
  <rationals>
    <delta>
      <rational>
        <numerator>1</numerator>
        <denominator>64</denominator>
      </rational>
    </delta>
  </rationals>
</xsl:template>

<!-- BEGIN: renamings -->
<xsl:template match="argument">
  <xsl:apply-templates select="*">
    <collapsing><xsl:copy-of select="."/></collapsing>
  </xsl:apply-templates>
</xsl:template>

<xsl:template match="reverse">
  <xsl:choose>
    <xsl:when test="//certificationProblem/goal/termination">
      <trsTerminationProof>
        <stringReversal>
          <xsl:apply-templates select="trs"/>
          <xsl:apply-templates select="*[last()]"/>
        </stringReversal>
      </trsTerminationProof>
    </xsl:when>
    <xsl:when test="//certificationProblem/goal/nonTermination">
      <trsNonterminationProof>
        <stringReversal>
          <xsl:apply-templates select="trs"/>
          <xsl:apply-templates select="*[last()]"/>
        </stringReversal>
      </trsNonterminationProof>
    </xsl:when>
  </xsl:choose>
</xsl:template>

<xsl:template match="linearPolynomial">
  <polynomial>
    <xsl:apply-templates select="*"/>
  </polynomial>
</xsl:template>

<xsl:template match="projEntry">
  <argumentFilterEntry>
    <xsl:apply-templates select="*"/>
  </argumentFilterEntry>
</xsl:template>

<xsl:template match="simpleProjection">
  <argumentFilter>
    <xsl:apply-templates select="*"/>
  </argumentFilter>
</xsl:template>

<xsl:template match="spscProc">
  <dpProof>
    <subtermProc>
      <xsl:apply-templates select="*[1]"/>
      <xsl:apply-templates select="*[2]"/>
      <xsl:apply-templates select="*[last()]"/>
    </subtermProc>
  </dpProof>
</xsl:template>

<xsl:template match="sizeChangeCriterion">
  <dpProof>
    <sizeChangeProc>
      <xsl:apply-templates select="*"/>
    </sizeChangeProc>
  </dpProof>
</xsl:template>
<!-- END -->

<xsl:template match="symbol">
  <xsl:call-template name="symbol">
    <xsl:with-param name="args" select="*"/>
  </xsl:call-template>
</xsl:template>

<xsl:template name="symbol">
  <xsl:param name="args"/>
  <xsl:choose>
    <xsl:when test="count($args) = 1">
      <xsl:copy-of select="$args[1]"/>
    </xsl:when>
    <xsl:when test="$args[last()] = sharp">
      <sharp>
        <xsl:call-template name="symbol">
          <xsl:with-param name="args" select="$args[position() != last()]"/>
        </xsl:call-template>
      </sharp>
    </xsl:when>
    <xsl:when test="$args[last()] = numberLabel">
      <labeledSymbol>
        <xsl:call-template name="symbol">
          <xsl:with-param name="args" select="$args[position() != last()]"/>
        </xsl:call-template>
        <numberLabel>
        <xsl:apply-templates select="$args[last()]/*"/>
        </numberLabel>
      </labeledSymbol>
    </xsl:when>
    <xsl:when test="$args[last()] = symbolLabel">
      <labeledSymbol>
        <xsl:call-template name="symbol">
          <xsl:with-param name="args" select="$args[position() != last()]"/>
        </xsl:call-template>
        <symbolLabel>
        <xsl:apply-templates select="$args[last()]/*"/>
        </symbolLabel>
      </labeledSymbol>
    </xsl:when>
    <xsl:otherwise/>
  </xsl:choose>
</xsl:template>

<xsl:template name="redPairProc">
  <xsl:param name="UR"/>
  <xsl:param name="P"/>
  <dpProof>
  <xsl:choose>
    <xsl:when test="$P/usableRules/rules and count($P/usableRules/rules) > 0">
      <redPairUrProc>
        <xsl:apply-templates select="$P/redPair"/>
        <xsl:apply-templates select="$P/dps"/>
        <xsl:apply-templates select="$P/usableRules"/>
        <xsl:apply-templates select="$P/*[last()]"/>
      </redPairUrProc>
    </xsl:when>
    <xsl:when test="$UR and count($UR) > 0">
      <redPairUrProc>
        <xsl:apply-templates select="$P/redPair"/>
        <xsl:apply-templates select="$P/dps"/>
        <usableRules>
          <xsl:apply-templates select="$UR"/>
        </usableRules>
        <xsl:apply-templates select="$P/*[last()]"/>
      </redPairUrProc>
    </xsl:when>
    <xsl:otherwise>
      <redPairProc>
        <xsl:apply-templates select="$P/redPair"/>
        <xsl:apply-templates select="$P/dps"/>
        <xsl:apply-templates select="$P/*[last()]"/>
      </redPairProc>
    </xsl:otherwise>
  </xsl:choose>
  </dpProof>
</xsl:template>

<xsl:template match="redPairProc">
  <xsl:call-template name="redPairProc">
    <xsl:with-param name="P" select="current()"/>
  </xsl:call-template>
</xsl:template>

<xsl:template name="monoRedPairProc">
  <xsl:param name="UR"/>
  <xsl:param name="P"/>
  <dpProof>
  <xsl:choose>
    <xsl:when test="$P/usableRules/rules and count($P/usableRules/rules) > 0">
      <monoRedPairUrProc>
        <xsl:apply-templates select="$P/redPair"/>
        <xsl:apply-templates select="$P/dps"/>
        <xsl:apply-templates select="$P/trs"/>
        <xsl:apply-templates select="$P/usableRules"/>
        <xsl:apply-templates select="$P/*[last()]"/>
      </monoRedPairUrProc>
    </xsl:when>
    <xsl:when test="$UR and count($UR) > 0">
      <monoRedPairUrProc>
        <xsl:apply-templates select="$P/redPair"/>
        <xsl:apply-templates select="$P/dps"/>
        <xsl:apply-templates select="$P/trs"/>
        <usableRules>
          <xsl:apply-templates select="$UR"/>
        </usableRules>
        <xsl:apply-templates select="$P/*[last()]"/>
      </monoRedPairUrProc>
    </xsl:when>
    <xsl:otherwise>
      <monoRedPairProc>
        <xsl:apply-templates select="$P/redPair"/>
        <xsl:apply-templates select="$P/dps"/>
        <xsl:apply-templates select="$P/trs"/>
        <xsl:apply-templates select="$P/*[last()]"/>
      </monoRedPairProc>
    </xsl:otherwise>
  </xsl:choose>
  </dpProof>
</xsl:template>

<xsl:template match="monoRedPairProc">
  <xsl:call-template name="monoRedPairProc">
    <xsl:with-param name="P" select="current()"/>
  </xsl:call-template>
</xsl:template>

<xsl:template match="usableRulesProc">
  <xsl:choose>
    <xsl:when test="*[last()] = redPairProc">
      <xsl:call-template name="redPairProc">
        <xsl:with-param name="UR" select="trs/rules"/>
        <xsl:with-param name="P" select="*[last()]"/>
      </xsl:call-template>
    </xsl:when>
    <xsl:when test="*[last()] = monoRedPairProc">
      <xsl:call-template name="monoRedPairProc">
        <xsl:with-param name="UR" select="trs/rules"/>
        <xsl:with-param name="P" select="*[last()]"/>
      </xsl:call-template>
    </xsl:when>
    <xsl:otherwise/>
  </xsl:choose>
</xsl:template>

<xsl:template match="dpTrans">
  <trsTerminationProof>
  <dpTrans>
    <xsl:apply-templates select="*[1]"/>
    <markedSymbols>true</markedSymbols>
    <xsl:apply-templates select="*[last()]"/>
  </dpTrans>
  </trsTerminationProof>
</xsl:template>

<xsl:template match="restore">
  <xsl:apply-templates select="*[last()]"/>
</xsl:template>

<xsl:template match="dependencyGraph">
  <xsl:choose>
    <xsl:when test="*[last()] = pIsEmpty">
      <dpProof>
      <depGraphProc>
        <xsl:for-each select="./dps/rules/rule">
          <component>
            <dps>
              <rules>
                <xsl:apply-templates select="."/>
              </rules>
            </dps>
            <realScc>false</realScc>
          </component>
        </xsl:for-each>
      </depGraphProc>
      </dpProof>
    </xsl:when>
    <xsl:otherwise>
      <xsl:apply-templates select="*[last()]"/>
    </xsl:otherwise>
  </xsl:choose>
</xsl:template>

</xsl:stylesheet>
