<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet 
    xmlns:xsl="http://www.w3.org/1999/XSL/Transform" 
    xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
    version="1.0">

    <xsl:output method="xml" indent="yes"/>
    <xsl:strip-space elements="*"/>

<xsl:template match="certificationProblem">
  <xsl:processing-instruction
    name="xml-stylesheet">type="text/xsl" href="cpfHTML.xsl"
  </xsl:processing-instruction>
  <xsl:copy>  
    <xsl:attribute
      name="xsi:noNamespaceSchemaLocation">cpf.xsd</xsl:attribute>
    <xsl:apply-templates select="input"/>
    <cpfVersion>2.0</cpfVersion>
    <xsl:apply-templates select="proof"/>
    <xsl:apply-templates select="origin"/>
  </xsl:copy>        
</xsl:template>
    
<!-- loops are the only complicated thing 
     here, we have to compute C[t sigma]   -->
<xsl:template match="funContext" mode="lastTerm">
    <funapp>
        <xsl:apply-templates select="*[1]"/>             <!-- copy symbol -->
        <xsl:for-each select="before/*">
            <arg>
                <xsl:apply-templates select="."/>                    
            </arg>
        </xsl:for-each>
        <arg>
            <xsl:apply-templates select="*[3]" mode="lastTerm"/>
        </arg>
        <xsl:for-each select="after/*">
            <arg>
                <xsl:apply-templates select="."/>                    
            </arg>
        </xsl:for-each>
    </funapp>
</xsl:template>

<xsl:template match="box" mode="lastTerm">
    <xsl:apply-templates select="//loop/redex[1]/*[1]" mode="lastTerm"/>   <!-- fetch initial term -->
</xsl:template>

<xsl:template match="funapp" mode="lastTerm">
    <funapp>
        <xsl:apply-templates select="*[1]"/> <!-- copy symbol -->
        <xsl:for-each select="arg/*">
            <arg>
                <xsl:apply-templates select="." mode="lastTerm"/>
            </arg>
        </xsl:for-each>
    </funapp>
</xsl:template>

<xsl:template match="var" mode="lastTerm">
    <xsl:variable name="x" select="text()"/>
    <xsl:choose>
        <xsl:when test="//loop/substitution/substEntry[*[1]/text() = $x]">
            <xsl:apply-templates select="//loop/substitution/substEntry[*[1]/text() = $x]/*[2]"/>
        </xsl:when>
        <xsl:otherwise>
            <xsl:apply-templates select="."/>
        </xsl:otherwise>
    </xsl:choose>
</xsl:template>        

<xsl:template name="step">
    <xsl:param name="i"/>
    <xsl:param name="n"/>
    <rewriteStep>
        <xsl:apply-templates select="redex[$i]/positionInTerm"/>
        <xsl:apply-templates select="redex[$i]/rule"/>
        <xsl:choose>
            <xsl:when test="$i = $n">
                <xsl:apply-templates select="*[last()]" mode="lastTerm"/>
            </xsl:when>
            <xsl:otherwise>
                <xsl:apply-templates select="redex[$i+1]/*[1]"/>                    
            </xsl:otherwise>
        </xsl:choose>            
    </rewriteStep>
    <xsl:if test="$i &lt; $n">
        <xsl:call-template name="step">
            <xsl:with-param name="i" select="$i + 1"/>
            <xsl:with-param name="n" select="$n"/>
        </xsl:call-template>
    </xsl:if>
</xsl:template>

<xsl:template match="loop">
    <trsNonterminationProof>
        <xsl:copy>
            <rewriteSequence>
                <startTerm>
                    <xsl:apply-templates select="redex[1]/*[1]"/>
                </startTerm>
                <xsl:call-template name="step">
                    <xsl:with-param name="i" select="1"/>
                    <xsl:with-param name="n" select="count(redex)"/>
                </xsl:call-template>
            </rewriteSequence>
            <xsl:apply-templates select="substitution"/>
            <xsl:apply-templates select="*[last()]"/> <!-- copy context -->
        </xsl:copy>
    </trsNonterminationProof>
</xsl:template>

<xsl:template match="*">
    <xsl:copy>
        <xsl:copy-of select="@*"/>
        <xsl:apply-templates/>
    </xsl:copy>
</xsl:template>    

</xsl:stylesheet>

